TopTreeAction

Trait TopTreeAction 

pub trait TopTreeAction<S>
where S: TopTreeSpec,
{ type Action: PartialEq; type ActionMonoid: Monoid<T = Self::Action>; const ROOT_TO_NODE_TOP_DOWN: bool = true; // Required methods fn act_info(info: &mut S::Info, action: &Self::Action); fn act_point(point: &mut S::Point, action: &Self::Action); fn act_path(path: &mut S::Path, action: &Self::Action); fn act_path_light(path: &mut S::Path, action: &Self::Action); }
act_path applies the heavy contribution and act_path_light applies the light contribution represented in the same path aggregate.

Provided Associated Constants

const ROOT_TO_NODE_TOP_DOWN: bool = true

Whether pending actions require propagation from the auxiliary root before rotations.

Required Associated Types

type Action: PartialEq

type ActionMonoid: Monoid<T = Self::Action>

Composes actions in application order: operate(a, b) applies a before b.

Required Methods

fn act_info(info: &mut S::Info, action: &Self::Action)

fn act_point(point: &mut S::Point, action: &Self::Action)

fn act_path(path: &mut S::Path, action: &Self::Action)

fn act_path_light(path: &mut S::Path, action: &Self::Action)

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors